@@ -12,7 +12,8 @@ from understory.web.framework.util import tx
 
 app = web.application("Canopy", static=__name__, year=r"\d{4}", month=r"\d{2}",
                       day=r"\d{2}", seconds=web.nb60_re + r"{,4}",
-                      slug=r"[\w_-]+", topic=r".+", person=r".+", event=".+")
+                      slug=r"[\w_-]+", room=r".+", topic=r".+",
+                      person=r".+", event=".+")
 tmpl = web.templates(__name__)
 
 
@@ -20,24 +21,21 @@ tmpl = web.templates(__name__)
 class Home:
     """."""
 
-    mentionable = True
-
-    def _get(self):
-        about = self.delegate(Me)
+    def get(self):
         recent = self.delegate(Recent)
-        return tmpl.home(about, recent)
+        return tmpl.home(recent)
 
 
 @app.route(r"postjob")
 class PostJob:
     """."""
 
-    def _get(self):
-        web.enqueue(_foobar, 3)
+    def get(self):
+        web.enqueue(foobar, 3)
         return "ok"
 
 
-def _foobar(n):
+def foobar(n):
     time.sleep(5)
     print(f"batbaz {n}")
 
@@ -46,16 +44,14 @@ def _foobar(n):
 class Me:
     """."""
 
-    mentionable = True
-
-    def _get(self):
+    def get(self):
         try:
             me = tx.pub.read("people/me")["resource"]
         except IndexError:
             raise web.OK(tmpl.new())
         return tmpl.about(me)
 
-    def _post(self):
+    def post(self):
         profile = web.form()
         profile["urls"] = profile["urls"].splitlines()
         profile["type"] = "card"
@@ -68,10 +64,8 @@ class Me:
 class Recent:
     """."""
 
-    mentionable = True
-
-    def _get(self):
-        recent = tx.pub.recent_entries()
+    def get(self):
+        recent = tx.pub.get_entries()
         return tmpl.recent(recent)
 
 
@@ -79,9 +73,7 @@ class Recent:
 class Now:
     """."""
 
-    mentionable = True
-
-    def _get(self):
+    def get(self):
         try:
             now = tx.pub.read("now")["resource"]
         except IndexError:
@@ -95,7 +87,7 @@ class Now:
 class NowPeers:
     """Firehose of peers' /now braids."""
 
-    def _subscribe(self):
+    def subscribe(self):
         return web.multi_subscribe("alice.com/now", "bob.com/now")
         # for patch in web.multi_subscribe(*peers):
         #     yield bytes(f"proxying: {patch.decode()}", "utf-8")
@@ -105,15 +97,15 @@ class NowPeers:
 class ArchiveYear:
     """Resources from given year."""
 
-    def _get(self):
-        return tmpl.archive.year()
+    def get(self):
+        return tmpl.archive.year(self.year, tx.pub.get_year(self.year))
 
 
 @app.route(r"{year}/{month}")
 class ArchiveMonth:
     """Resources from given month."""
 
-    def _get(self):
+    def get(self):
         return tmpl.archive.month()
 
 
@@ -121,7 +113,7 @@ class ArchiveMonth:
 class ArchiveDay:
     """Resources from given day."""
 
-    def _get(self):
+    def get(self):
         return tmpl.archive.day()
 
 
@@ -129,7 +121,7 @@ class ArchiveDay:
 class Tracks:
     """Coords."""
 
-    def _post(self):
+    def post(self):
         print()
         pprint(json.loads(web.form()))
         print()
@@ -139,20 +131,27 @@ class Tracks:
 class Entry:
     """An individual entry."""
 
-    mentionable = True
-
-    def _get(self):
+    def get(self):
         resource = tx.pub.read(tx.request.uri.path)["resource"]
         if resource["visibility"] == "private" and not tx.user.session:
-            raise web.SeeOther(f"/sign-in?return_to={tx.request.uri.path}")
-        return tmpl.entry(resource)
+            raise web.Unauthorized(f"/auth?return_to={tx.request.uri.path}")
+        mentions = web.indie.webmention.get_mentions(str(tx.request.uri))
+        return tmpl.entry(resource, mentions)
+
+
+@app.route(r"rooms/{room}")
+class Room:
+    """A chat room for given topic."""
+
+    def get(self):
+        return tmpl.room(self.room)
 
 
 @app.route(r"chat/{topic}")
 class ChatTopic:
     """A chat room for given topic."""
 
-    def _get(self):
+    def get(self):
         return tmpl.chat(self.topic)
 
 
@@ -160,7 +159,7 @@ class ChatTopic:
 class Network:
     """Your social network."""
 
-    def _get(self):
+    def get(self):
         resource = tx.pub.read(tx.request.uri.path)
         return tmpl.resource(resource)
 
@@ -169,7 +168,7 @@ class Network:
 class Person:
     """A person in your network."""
 
-    def _get(self):
+    def get(self):
         resource = tx.pub.read(tx.request.uri.path)
         return tmpl.resource(resource)
 
@@ -178,7 +177,7 @@ class Person:
 class Calendar:
     """Your event calendar."""
 
-    def _get(self):
+    def get(self):
         resource = tx.pub.read(tx.request.uri.path)
         return tmpl.resource(resource)
 
@@ -187,7 +186,7 @@ class Calendar:
 class Event:
     """An event on your calendar."""
 
-    def _get(self):
+    def get(self):
         resource = tx.pub.read(tx.request.uri.path)
         return tmpl.resource(resource)
 
@@ -196,7 +195,7 @@ class Event:
 class IconEditor:
     """An icon editor."""
 
-    def _get(self):
+    def get(self):
         icons = {"bookmark": (576, 512, """
             M 144,32
             C 136,32 128,40 128,48
@@ -213,15 +212,13 @@ class IconEditor:
 class Initialize:
     """."""
 
-    def _post(self):
+    def post(self):
         name = web.form("name").name
-        uid = str(web.uri(tx.origin))
-        tx.pub.create("card", slug="me", name=name, uid=uid, url=[uid])
+        uid, passphrase = web.indieauth.add_owner(name)
         tx.pub.create("entry", content="Hello world!")
-        salt, scrypt_hash, passphrase = web.generate_passphrase()
-        tx.db.insert("credentials", salt=salt, scrypt_hash=scrypt_hash)
-        tx.user.session["me"] = uid
-        tx.owner = tx.pub.read("people/me")["resource"]
+        tx.user.session["uid"] = uid
+        tx.user.session["name"] = name
+        tx.host.owner = tx.pub.read(uid)["resource"]
         return tmpl.welcome(passphrase)
 
 
@@ -229,37 +226,20 @@ class Initialize:
 class Settings:
     """."""
 
-    def _get(self):
+    def get(self):
         return tmpl.settings()
 
-    def _post(self):
+    def post(self):
         form = web.form("theme")
         print(form.theme)
         raise web.SeeOther("/settings")
 
 
-@app.route(r"sign-in")
-class SignIn:
-    """Sign in as the owner of the site."""
-
-    def _get(self):
-        return tmpl.sign_in()
-
-    def _post(self):
-        form = web.form("passphrase", return_to="/")
-        credential = tx.db.select("credentials", order="created DESC")[0]
-        if web.verify_passphrase(credential["salt"], credential["scrypt_hash"],
-                                 form.passphrase):
-            tx.user.session["me"] = tx.owner["uid"]
-            raise web.SeeOther(form.return_to)
-        raise web.Unauthorized("bad passphrase")
-
-
 @app.route(r"debug")
 class Debug:
     """Client-side debugging."""
 
-    def _get(self):
+    def get(self):
         return "debug log"
 
 
@@ -267,6 +247,7 @@ app.mount(web.cache_app)
 # TODO app.mount(web.debug_app)
 app.mount(web.indieauth.client)
 app.mount(web.indieauth.server)
+app.mount(web.indieauth.root)
 app.mount(web.micropub.server)
 app.mount(web.microsub.server)
 app.mount(web.webmention.receiver)
@@ -293,33 +274,33 @@ def contextualize(handler, app):
     yield
 
 
-app.wrap(web.braidify)
 app.wrap(web.resume_session)
+app.wrap(web.braidify)
 
 
 @app.wrap
 def template(handler, app):
     """Wrap the response in a template."""
-    tx.user.is_owner = tx.user.session.get("me") == str(web.uri(tx.origin))
     yield
-    if tx.response.headers.content_type == "text/html":
-        tx.response.body = tmpl.template(tx.owner, tx.response.body)
+    if tx.response.headers.content_type == "text/html" \
+       and not tx.response.naked and "owner" in tx.host:
+        tx.response.body = tmpl.template(tx.host.owner, tx.response.body)
 
 
-app.wrap(web.indieauth.wrap_client, "post")
-app.wrap(web.indieauth.wrap_server, "post")
 app.wrap(web.micropub.wrap_server, "post")
-app.wrap(web.microsub.wrap_server, "post")
+app.wrap(web.indieauth.wrap_server, "post")
+app.wrap(web.indieauth.wrap_client, "post")
 app.wrap(web.webmention.wrap, "post")
+app.wrap(web.microsub.wrap_server, "post")
 app.wrap(web.websub.wrap, "post")
+app.wrap(web.micropub.route_unrouted, "post")
 
 
+# TODO XXX move to indieauth module
 def finalize(handler, app):
     """Leverage the IndieWeb tools above for use during this transaction."""
-    try:
-        tx.owner = tx.pub.read("people/me")["resource"]
-    except IndexError:
-        tx.owner = None
+    if "owner" not in tx.host and tx.request.method == "GET":
+        raise web.OK(tmpl.new())
     yield
 
 
